/*
 * Name:    mp_image.c
 *
 * Purpose: Python module interface to generic MX image functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2006-2007, 2009, 2011 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "mp.h"

#include "mx_array.h"

MX_EXPORT mx_status_type
MpPrivate_get_image_frame_from_handle( long handle,
				MX_IMAGE_FRAME **image_frame )
{
	static const char fname[] = "MpPrivate_get_image_frame_from_handle()";

	signed long image_frame_handle;
	void *pointer;
	mx_status_type mx_status;

	if ( image_frame == NULL ) {
		return mx_error( MXE_NULL_ARGUMENT, fname,
		"NULL pointer to MX_IMAGE_FRAME pointer passed." );
	}

	*image_frame = NULL;

	image_frame_handle = (signed long) handle;

	/* Convert the handle to an MX_IMAGE_FRAME pointer. */

	if ( image_frame_handle < 0 ) {
		*image_frame = NULL;
	} else {
		mx_status = mx_get_pointer_from_handle( &pointer,
				MpPrivate_global_image_frame_handle_table,
				image_frame_handle );

		if ( mx_status.code != MXE_SUCCESS )
			return mx_status;

		*image_frame = (MX_IMAGE_FRAME *) pointer;
	}

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT PyObject *
MpPrivate_image_init_cmd( PyObject *self, PyObject *args )
{
	static const char cname[] = "image_init";

	PyObject *result, *framesize_sequence, *framesize_item;
	MX_IMAGE_FRAME *image_frame;
	long framesize[2];
	long i, image_format, pixel_order;
	long header_length, image_length, image_frame_handle;
	double bytes_per_pixel;
	int parse_status, num_elements;
	mx_status_type mx_status;

	/* Create an image frame handle table if this has not yet been done. */

	if ( MpPrivate_global_image_frame_handle_table == NULL ) {

		/* Create a global handle table for the image frames.
		 * Configure the table to be incremented in blocks of
		 * 100 handles.
		 */

		mx_status = mx_create_handle_table(
				&MpPrivate_global_image_frame_handle_table,
				100, 1 );

		if ( mx_status.code != MXE_SUCCESS )
			return MpError_SetFromMxStatus( NULL, mx_status );
	}

	/* Parse the method arguments. */

	parse_status = PyArg_ParseTuple( args, "Olldll",
					&framesize_sequence,
					&image_format,
					&pixel_order,
					&bytes_per_pixel,
					&header_length,
					&image_length );

	if ( parse_status == 0 )
		return NULL;

	num_elements = PySequence_Length( framesize_sequence );

	if ( num_elements != 2 ) {
		MpError_SetString( MpPrivateExc_Illegal_Argument,
	    "%s: The framesize sequence passed had %d elements which is "
	    "incorrect.  Instead, the sequence always must have 2 elements.",
			cname, num_elements );

		return NULL;
	}

	/* Get the width and height of the frame from the framesize sequence. */

	for ( i = 0; i < 2; i++ ) {
		framesize_item = PySequence_GetItem( framesize_sequence, i );

		framesize[i] = PyInt_AsLong( framesize_item );
	}

	/* Create a new image frame.
	 *
	 * Setting image_frame to NULL beforehand tells mx_image_alloc()
	 * to create a completely new image frame rather than trying
	 * to reuse an old one.
	 */

	image_frame = NULL;

	mx_status = mx_image_alloc( &image_frame,
					framesize[0],
					framesize[1],
					image_format,
					pixel_order,
					bytes_per_pixel,
					header_length,
					image_length );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Create a handle for the image frame. */

	mx_status = mx_create_handle( &image_frame_handle,
				MpPrivate_global_image_frame_handle_table,
				image_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Return the handle to Python. */

	result = Py_BuildValue( "l", image_frame_handle );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_image_del_cmd( PyObject *self, PyObject *args )
{
	MX_IMAGE_FRAME *image_frame;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_image_frame_from_handle( handle,
							&image_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_image_free( image_frame );

	mx_status = mx_delete_handle( handle,
				MpPrivate_global_image_frame_handle_table );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_image_get_image_format_type_from_name_cmd( PyObject *self,
							PyObject *args )
{
	PyObject *result;
	long format_type;
	char *format_name;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "s", &format_name );

	if ( parse_status == 0 )
		return NULL;

	mx_status = mx_image_get_image_format_type_from_name( format_name,
								&format_type );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", format_type );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_image_get_image_format_name_from_type_cmd( PyObject *self,
							PyObject *args )
{
	PyObject *result;
	long format_type;
	char format_name[MXU_IMAGE_FORMAT_NAME_LENGTH];
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &format_type );

	if ( parse_status == 0 )
		return NULL;

	mx_status = mx_image_get_image_format_name_from_type( format_type,
					format_name, sizeof(format_name) );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "s", format_name );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_image_get_file_format_type_from_name_cmd( PyObject *self,
							PyObject *args )
{
	PyObject *result;
	long format_type;
	char *format_name;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "s", &format_name );

	if ( parse_status == 0 )
		return NULL;

	mx_status = mx_image_get_file_format_type_from_name( format_name,
								&format_type );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", format_type );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_image_get_file_format_name_from_type_cmd( PyObject *self,
							PyObject *args )
{
	PyObject *result;
	long format_type;
	char format_name[MXU_IMAGE_FORMAT_NAME_LENGTH];
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &format_type );

	if ( parse_status == 0 )
		return NULL;

	mx_status = mx_image_get_file_format_name_from_type( format_type,
					format_name, sizeof(format_name) );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "s", format_name );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_image_get_exposure_time_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_IMAGE_FRAME *image_frame;
	long handle;
	double exposure_time;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_image_frame_from_handle( handle,
							&image_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_image_get_exposure_time( image_frame, &exposure_time );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", exposure_time );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_image_get_image_data_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_IMAGE_FRAME *image_frame;
	long handle;
	size_t image_length;
	void *image_data_pointer;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_image_frame_from_handle( handle,
							&image_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_image_get_image_data_pointer( image_frame,
						&image_length,
						&image_data_pointer );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "s#",
			(char *) image_data_pointer, image_length );

	return result;
}

/* FIXME: Several methods are missing here.  See Mp.py for the details. */

MX_EXPORT PyObject *
MpPrivate_image_read_file_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_IMAGE_FRAME *image_frame;
	long handle, datafile_type;
	char *datafile_name;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lls",
				&handle, &datafile_type, &datafile_name );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_image_frame_from_handle( handle,
							&image_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_image_read_file( &image_frame,
					datafile_type, datafile_name );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "(l,l)lldll",
			MXIF_ROW_FRAMESIZE(image_frame),
			MXIF_COLUMN_FRAMESIZE(image_frame),
			MXIF_IMAGE_FORMAT(image_frame),
			MXIF_BYTE_ORDER(image_frame),
			MXIF_BYTES_PER_PIXEL(image_frame),
			(long) image_frame->header_length,
			(long) image_frame->image_length );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_image_write_file_cmd( PyObject *self, PyObject *args )
{
	MX_IMAGE_FRAME *image_frame;
	long handle, datafile_type;
	char *datafile_name;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lls",
				&handle, &datafile_type, &datafile_name );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_image_frame_from_handle( handle,
							&image_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_image_write_file( image_frame,
					datafile_type, datafile_name );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

