/*
 * Name:    mp_motor.c
 *
 * Purpose: Python module interface to generic MX motor functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2002-2003, 2005-2006, 2009, 2011 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "mp.h"

#include "mx_motor.h"

MX_EXPORT PyObject *
MpPrivate_motor_is_busy_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_bool_type busy;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_is_busy( record, &busy );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", (int) busy );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_move_absolute_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double destination;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld", &handle, &destination );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_move_absolute( record, destination, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_move_relative_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double relative_position;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld",
						&handle, &relative_position );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_move_relative( record, relative_position, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_move_absolute_with_flags_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long flags;
	double destination;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ldl",
					&handle, &destination, &flags );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_move_absolute( record, destination, flags );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_move_relative_with_flags_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long flags;
	double relative_position;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ldl",
					&handle, &relative_position, &flags );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_move_relative( record, relative_position, flags );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_wait_for_motor_stop_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long flags;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &flags );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_wait_for_motor_stop( record, flags );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_position_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double position;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_position( record, &position );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", position );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_position_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double set_position;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld", &handle, &set_position );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_set_position( record, set_position );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_soft_abort_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_soft_abort( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_immediate_abort_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_immediate_abort( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_positive_limit_hit_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_bool_type limit_hit;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_positive_limit_hit( record, &limit_hit );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", (int) limit_hit );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_negative_limit_hit_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_bool_type limit_hit;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_negative_limit_hit( record, &limit_hit );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", (int) limit_hit );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_find_home_position_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long direction;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &direction );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_find_home_position( record, (int) direction );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_constant_velocity_move_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long direction;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &direction );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_constant_velocity_move( record, direction );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_status_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long motor_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_status( record, &motor_status );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", motor_status );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_extended_status_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double motor_position;
	unsigned long motor_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_extended_status( record,
					&motor_position, &motor_status );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* The extended status is returned as a two element tuple. */

	result = Py_BuildValue( "dl", motor_position, motor_status );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_speed_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double speed;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_speed( record, &speed );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", speed );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_speed_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double speed;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld", &handle, &speed );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_set_speed( record, speed );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_base_speed_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double speed;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_base_speed( record, &speed );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", speed );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_base_speed_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double speed;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld", &handle, &speed );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_set_base_speed( record, speed );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_maximum_speed_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double speed;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_maximum_speed( record, &speed );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", speed );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_maximum_speed_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double speed;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld", &handle, &speed );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_set_maximum_speed( record, speed );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_save_speed_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_save_speed( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_restore_speed_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_restore_speed( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_speed_between_positions_cmd( PyObject *self,
							PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double position1, position2, time_for_move;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lddd", &handle,
				&position1, &position2, &time_for_move );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_set_speed_between_positions( record,
						position1, position2,
						time_for_move );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_accel_type_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long acceleration_type;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_acceleration_type( record,
							&acceleration_type );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", acceleration_type );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_accel_time_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double acceleration_time;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_acceleration_time( record,
						&acceleration_time );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", acceleration_time );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_accel_time_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double acceleration_time;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld",
					&handle, &acceleration_time );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_set_acceleration_time( record,
						acceleration_time );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_accel_distance_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double acceleration_distance;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_acceleration_distance( record,
						&acceleration_distance );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", acceleration_distance );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_raw_accel_parameters_cmd( PyObject *self, PyObject *args )
{
	PyObject *parameters_tuple, *parameters_element;
	MX_RECORD *record;
	long handle;
	int i, parse_status;
	double parameter_array[MX_MOTOR_NUM_ACCELERATION_PARAMS];
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_raw_acceleration_parameters(
						record, parameter_array );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Create a new tuple to contain the acceleration parameters. */

	parameters_tuple = PyTuple_New( MX_MOTOR_NUM_ACCELERATION_PARAMS );

	if ( parameters_tuple == NULL )
		return NULL;

	/* Copy the acceleration parameters to the tuple. */

	for ( i = 0; i < MX_MOTOR_NUM_ACCELERATION_PARAMS; i++ ) {

		parameters_element = PyFloat_FromDouble( parameter_array[i] );

		if ( parameters_element == NULL ) {
			Py_DECREF( parameters_tuple );
			return NULL;
		}

		PyTuple_SET_ITEM( parameters_tuple, i, parameters_element );
	}

	return parameters_tuple;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_raw_accel_parameters_cmd( PyObject *self, PyObject *args )
{
	PyObject *parameters_tuple, *parameters_element;
	MX_RECORD *record;
	long handle;
	int i, parse_status, type_status, element_status, tuple_length;
	double parameter_array[MX_MOTOR_NUM_ACCELERATION_PARAMS];
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lO",
					&handle, &parameters_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Verify that the object we have been passed is a tuple. */

	type_status = PyTuple_Check( parameters_tuple );

	if ( type_status == 0 )
		return NULL;

	/* Find out how many object are in the tuple. */

	tuple_length = PyTuple_Size( parameters_tuple );

	if ( tuple_length < 0 )
		return NULL;

	/* Check to see if we have received the correct number of parameters
	 * in the parameter object.
	 */

	if ( tuple_length != MX_MOTOR_NUM_ACCELERATION_PARAMS ) {

		/* Raise an exception if we have the wrong number
		 * of parameters.
		 */

		MpError_SetString( PyExc_IndexError,
			"The method '%s' was passed the wrong number of "
			"parameters, namely %d.  "
			"There should be %d parameters.",
			tuple_length, MX_MOTOR_NUM_ACCELERATION_PARAMS );

		return NULL;
	}

	/* Walk through the tuple and get the values of the individual
	 * parameters.
	 */

	for ( i = 0; i < MX_MOTOR_NUM_ACCELERATION_PARAMS; i++ ) {

		/* Get the next tuple element. */

		parameters_element = PyTuple_GetItem( parameters_tuple, i );

		if ( parameters_element == NULL )
			return NULL;

		/* Verify that the element is a PyFloatObject. */

		element_status = PyFloat_Check( parameters_element );

		if ( element_status == 0 )
			return NULL;

		/* Copy the tuple element value to the parameter array. */

		parameter_array[i] = PyFloat_AsDouble( parameters_element );
	}

	mx_status = mx_motor_set_raw_acceleration_parameters(
						record, parameter_array );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_home_search_succeeded_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_bool_type home_search_succeeded;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_home_search_succeeded( record,
						&home_search_succeeded );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", (int) home_search_succeeded );

	return result;
}

static int
MpPrivate_motor_get_gain_type( const char *cname,
				PyObject *gain_type_object,
				int *gain_type )
{
	PyObject *repr_object;
	char *gain_type_string;
	char *repr_string;

	/* gain_type_object must be either a Python integer object
	 * or a Python string object.
	 */

	if ( PyInt_Check( gain_type_object ) ) {

		/* gain_type_object is an integer object, so all we need
		 * to do is extract the value of the integer.
		 */

		*gain_type = (int) PyInt_AS_LONG( gain_type_object );

		return 1;
	}

	if ( PyString_Check( gain_type_object ) ) {

		/* gain_type_object is a string object, so we must compare
		 * the string value with the allowed gain type names.
		 */

		gain_type_string = PyString_AsString( gain_type_object );

		if ( strcmp( gain_type_string, "proportional" ) == 0 ) {
			*gain_type = MXLV_MTR_PROPORTIONAL_GAIN;
		} else
		if ( strcmp( gain_type_string, "integral" ) == 0 ) {
			*gain_type = MXLV_MTR_INTEGRAL_GAIN;
		} else
		if ( strcmp( gain_type_string, "derivative" ) == 0 ) {
			*gain_type = MXLV_MTR_DERIVATIVE_GAIN;
		} else
		if ( strcmp( gain_type_string, "velocity_feedforward" ) == 0 ) {
			*gain_type = MXLV_MTR_VELOCITY_FEEDFORWARD_GAIN;
		} else
		if ( strcmp( gain_type_string, "acceleration_feedforward" )
									== 0 ) {
			*gain_type = MXLV_MTR_ACCELERATION_FEEDFORWARD_GAIN;
		} else
		if ( strcmp( gain_type_string, "integral_limit" ) == 0 ) {
			*gain_type = MXLV_MTR_INTEGRAL_LIMIT;
		} else
		if ( strcmp( gain_type_string, "extra" ) == 0 ) {
			*gain_type = MXLV_MTR_EXTRA_GAIN;
		} else {
			MpError_SetString( PyExc_RuntimeError,
			"The requested gain type '%s' for '%s' is not a "
			"recognized gain type.", gain_type_string, cname );

			return 0;
		}

		return 1;
	}

	repr_object = PyObject_Repr( gain_type_object );

	if ( repr_object == NULL ) {
		MpError_SetString( PyExc_RuntimeError,
		"The gain type for '%s' is expected to be either an integer "
		"object or a string object, but it is of neither type.  "
		"In addition, an attempt to get the string representation "
		"of the gain type failed for some reason.", cname );

		return 0;
	}

	repr_string = PyString_AsString( repr_object );

	MpError_SetString( PyExc_TypeError,
		"The gain type '%s' for '%s' is expected to be either "
		"an integer object or a string object, but it is "
		"of neither type.", repr_string, cname );

	Py_DECREF( repr_object );

	return 0;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_gain_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "get_gain";

	PyObject *result;
	PyObject *gain_type_object;
	MX_RECORD *record;
	long handle;
	int parse_status, gain_type;
	double gain;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lO",
					&handle, &gain_type_object );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	parse_status = MpPrivate_motor_get_gain_type( cname,
					gain_type_object, &gain_type );
	if ( parse_status == 0 )
		return NULL;

	mx_status = mx_motor_get_gain( record, gain_type, &gain );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", gain );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_gain_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "set_gain";

	PyObject *gain_type_object;
	MX_RECORD *record;
	long handle;
	int parse_status, gain_type;
	double gain;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lOd",
					&handle, &gain_type_object, &gain );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	parse_status = MpPrivate_motor_get_gain_type( cname,
					gain_type_object, &gain_type );
	if ( parse_status == 0 )
		return NULL;

	mx_status = mx_motor_set_gain( record, gain_type, gain );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

static int
MpPrivate_motor_get_command_type( const char *cname,
				PyObject *command_type_object,
				int *command_type )
{
	PyObject *repr_object;
	char *command_type_string;
	char *repr_string;

	/* command_type_object must be either a Python integer object
	 * or a Python string object.
	 */

	if ( PyInt_Check( command_type_object ) ) {

		/* command_type_object is an integer object, so all we need
		 * to do is extract the value of the integer.
		 */

		*command_type = (int) PyInt_AS_LONG( command_type_object );

		return 1;
	}

	if ( PyString_Check( command_type_object ) ) {

		/* command_type_object is a string object, so we must compare
		 * the string value with the allowed command type names.
		 */

		command_type_string = PyString_AsString( command_type_object );

		if ( strcmp( command_type_string, "axis_enable" ) == 0 ) {
			*command_type = MXLV_MTR_AXIS_ENABLE;
		} else
		if ( strcmp( command_type_string, "closed_loop" ) == 0 ) {
			*command_type = MXLV_MTR_CLOSED_LOOP;
		} else
		if ( strcmp( command_type_string, "fault_reset" ) == 0 ) {
			*command_type = MXLV_MTR_FAULT_RESET;
		} else {
			MpError_SetString( PyExc_RuntimeError,
			"The requested command type '%s' for '%s' is not a "
			"recognized command type.", command_type_string, cname);

			return 0;
		}

		return 1;
	}

	repr_object = PyObject_Repr( command_type_object );

	if ( repr_object == NULL ) {
		MpError_SetString( PyExc_RuntimeError,
		"The command type for '%s' is expected to be either an integer "
		"object or a string object, but it is of neither type.  "
		"In addition, an attempt to get the string representation "
		"of the command type failed for some reason.", cname );

		return 0;
	}

	repr_string = PyString_AsString( repr_object );

	MpError_SetString( PyExc_TypeError,
		"The command type '%s' for '%s' is expected to be either "
		"an integer object or a string object, but it is "
		"of neither type.", repr_string, cname );

	Py_DECREF( repr_object );

	return 0;
}

MX_EXPORT PyObject *
MpPrivate_motor_send_control_command_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "send_control_command";

	PyObject *command_type_object;
	MX_RECORD *record;
	long handle;
	int parse_status, command_type, command;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lOi",
				&handle, &command_type_object, &command );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	parse_status = MpPrivate_motor_get_command_type( cname,
					command_type_object, &command_type );
	if ( parse_status == 0 )
		return NULL;

	mx_status = mx_motor_send_control_command( record,
						command_type, command );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

/*** The following motors operate on multiple motors at once. ***/

#ifdef FREE_ARRAYS
#undef FREE_ARRAYS
#endif

#define FREE_ARRAYS \
		do { \
			mx_free( motor_record_array ); \
			mx_free( position_array ); \
		} while (0)

MX_EXPORT PyObject *
MpPrivate_multiple_motors_move_absolute_with_flags_cmd( PyObject *self,
							PyObject *args )
{
	const char cname[] = "multiple_motors_move_absolute_with_flags";

	PyObject *motors_tuple, *positions_tuple;
	PyObject *motor_object, *position_object;
	int i, parse_status, num_motors, num_positions;
	MX_RECORD **motor_record_array, *motor_record;
	double *position_array;
	unsigned long motion_flags;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "OOl",
			&motors_tuple, &positions_tuple, &motion_flags );

	if ( parse_status == 0 )
		return NULL;

	/* The first object passed should be a tuple of motor objects. */

	if ( PyTuple_Check( motors_tuple ) == 0 ) {
		MpError_SetString( PyExc_TypeError,
		"The first argument to '%s' should be a tuple, but it is not.",
			cname );
		return NULL;
	}

	/* The second object passed should be a tuple of motor positions. */

	if ( PyTuple_Check( positions_tuple ) == 0 ) {
		MpError_SetString( PyExc_TypeError,
		"The second argument to '%s' should be a tuple, but it is not.",
			cname );
		return NULL;
	}

	/* Figure out the number of motors by getting the number of objects
	 * in the motor tuple.
	 */

	num_motors = PyTuple_Size( motors_tuple );

	if ( num_motors <= 0 ) {
		MpError_SetString( PyExc_RuntimeError,
			"The motors tuple passed to '%s' was empty.", cname );
		return NULL;
	}

	/* The positions tuple should have the same number of objects as
	 * the motors tuple.
	 */

	num_positions = PyTuple_Size( positions_tuple );

	if ( num_positions != num_motors ) {
		MpError_SetString( PyExc_RuntimeError,
			"The two tuples passed to '%s' do not contain "
			"the same number of objects.", cname );
		return NULL;
	}

	/* Allocate memory for the motor record array. */

	motor_record_array = (MX_RECORD **) 
				malloc( num_motors * sizeof(MX_RECORD *) );

	if ( motor_record_array == NULL ) {
		MpError_SetString( PyExc_MemoryError,
			"%s: Unable to allocate a %d element array "
			"of MX_RECORD pointers.",
			cname, num_motors );
		return NULL;
	}

	/* Allocate memory for the position array. */

	position_array = (double *) malloc( num_motors * sizeof(double ) );

	if ( position_array == NULL ) {
		MpError_SetString( PyExc_MemoryError,
			"%s: Unable to allocate a %d element array of doubles.",
			cname, num_motors );

		free( motor_record_array );
		return NULL;
	}

	/* Fill in the motor record array from the motors tuple. */

	for ( i = 0; i < num_motors; i++ ) {
		motor_object = PyTuple_GetItem( motors_tuple, i );

		if ( motor_object == NULL ) {
			FREE_ARRAYS;
			return NULL;
		}

		mx_status = MpPrivate_get_record_from_object(
					motor_object, &motor_record );

		if ( mx_status.code != MXE_SUCCESS ) {
			FREE_ARRAYS;
			return MpError_SetFromMxStatus( NULL, mx_status );
		}
		motor_record_array[i] = motor_record;
	}

	/* Copy the position values to the position array. */

	for ( i = 0; i < num_motors; i++ ) {
		position_object = PyTuple_GetItem( positions_tuple, i );

		if ( position_object == NULL ) {
			FREE_ARRAYS;
			return NULL;
		}

		if ( PyFloat_Check( position_object ) == 0 ) {
			MpError_SetString( PyExc_TypeError,
	"%s: Element %d of the position tuple is not a floating point number.",
				cname, i );

			FREE_ARRAYS;
			return NULL;
		}
		position_array[i] = PyFloat_AsDouble( position_object );
	}

	/* Finally, we can start the motor moves. */

	mx_status = mx_motor_array_move_absolute( num_motors,
			motor_record_array, position_array, motion_flags );

	/* Before we do anything else, free the two arrays. */

	FREE_ARRAYS;

	/* If the motor move caused an error to occur, report that. */

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

#undef FREE_ARRAYS

MX_EXPORT PyObject *
MpPrivate_wait_for_multiple_motors_stop_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "wait_for_multiple_motors_stop";

	PyObject *motors_tuple;
	PyObject *motor_object;
	int i, parse_status, num_motors;
	MX_RECORD **motor_record_array, *motor_record;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "O", &motors_tuple );

	if ( parse_status == 0 )
		return NULL;

	/* The object passed should be a tuple of motor objects. */

	if ( PyTuple_Check( motors_tuple ) == 0 ) {
		MpError_SetString( PyExc_TypeError,
		"The argument to '%s' should be a tuple, but it is not.",
			cname );
		return NULL;
	}

	/* Figure out the number of motors by getting the number of objects
	 * in the motor tuple.
	 */

	num_motors = PyTuple_Size( motors_tuple );

	if ( num_motors <= 0 ) {
		MpError_SetString( PyExc_RuntimeError,
			"The motors tuple passed to '%s' was empty.", cname );
		return NULL;
	}

	/* Allocate memory for the motor record array. */

	motor_record_array = (MX_RECORD **) 
				malloc( num_motors * sizeof(MX_RECORD *) );

	if ( motor_record_array == NULL ) {
		MpError_SetString( PyExc_MemoryError,
			"%s: Unable to allocate a %d element array "
			"of MX_RECORD pointers.",
			cname, num_motors );
		return NULL;
	}

	/* Fill in the motor record array from the motors tuple. */

	for ( i = 0; i < num_motors; i++ ) {
		motor_object = PyTuple_GetItem( motors_tuple, i );

		if ( motor_object == NULL ) {
			mx_free( motor_record_array );
			return NULL;
		}

		mx_status = MpPrivate_get_record_from_object(
					motor_object, &motor_record );

		if ( mx_status.code != MXE_SUCCESS ) {
			mx_free( motor_record_array );
			return MpError_SetFromMxStatus( NULL, mx_status );
		}
		motor_record_array[i] = motor_record;
	}

	/* Finally, we are ready to wait for the motor moves to end. */

	mx_status = mx_wait_for_motor_array_stop( num_motors,
						motor_record_array, 0 );

	/* Before we do anything else, free the array. */

	mx_free( motor_record_array );

	/* If the motor move caused an error to occur, report that. */

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

