/*
 * Name:    mp_record.c
 *
 * Purpose: MX Python interface to generic MX record functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2002-2011 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "mp.h"

#include "mx_version.h"
#include "mx_driver.h"
#include "mx_net.h"
#include "mx_multi.h"
#include "mx_log.h"

MX_EXPORT mx_status_type
MpPrivate_get_record_from_handle( long handle,
				MX_RECORD **record )
{
	static const char fname[] = "MpPrivate_get_record_from_handle()";

	signed long record_handle;
	void *pointer;
	mx_status_type mx_status;

	if ( record == NULL ) {
		return mx_error( MXE_NULL_ARGUMENT, fname,
		"NULL pointer to MX_RECORD pointer passed." );
	}

	*record = NULL;

	record_handle = (signed long) handle;

	/* Convert it to an MX_RECORD pointer. */

	mx_status = mx_get_pointer_from_handle( &pointer,
					MpPrivate_global_record_handle_table,
					record_handle );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	*record = (MX_RECORD *) pointer;

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
MpPrivate_get_record_from_object( PyObject *object,
				MX_RECORD **record )
{
	static const char fname[] = "MpPrivate_get_record_from_object()";

	PyObject *handle_object;
	signed long record_handle;
	void *pointer;
	mx_status_type mx_status;

	if ( record == NULL ) {
		return mx_error( MXE_NULL_ARGUMENT, fname,
		"NULL pointer to MX_RECORD pointer passed." );
	}

	*record = NULL;

	/* Get the handle attribute object from the object passed to us. */

	if ( PyObject_HasAttrString( object, "handle" ) == 0 ) {
		return mx_error( MXE_TYPE_MISMATCH, fname,
			"The object passed does not have "
			"a handle attribute, so it cannot be "
			"an Mp.Record object." );
	}

	handle_object = PyObject_GetAttrString( object, "handle" );

	if ( handle_object == NULL ) {
		return mx_error( MXE_FUNCTION_FAILED, fname,
			"Even though the object passed is said to "
			"contain a handle attribute, an attempt to get "
			"that attribute failed.  This should never happen!" );
	}

	/* The handle object should contain a Python integer. */

	if ( PyInt_Check( handle_object ) == 0 ) {
		Py_DECREF( handle_object );

		return mx_error( MXE_TYPE_MISMATCH, fname,
			"The 'handle' attribute of the object passed "
			"is not a Python integer, so it cannot be "
			"an Mp.Record object." );
	}

	record_handle = (signed long) PyInt_AsLong( handle_object );

	Py_DECREF( handle_object );

	/* Convert it to an MX_RECORD pointer. */

	mx_status = mx_get_pointer_from_handle( &pointer,
					MpPrivate_global_record_handle_table,
					record_handle );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	*record = (MX_RECORD *) pointer;

	return MX_SUCCESSFUL_RESULT;
}

/*========*/


MX_EXPORT void
MpPrivate_print_object( PyObject *object )
{
	static const char fname[] = "MpPrivate_print_object()";

	PyObject *type_object, *repr_object;
	char *type_string, *repr_string;

	if ( object == (PyObject *) NULL ) {
		mx_error( MXE_NULL_ARGUMENT, fname,
		"The object pointer passed was NULL." );

		return;
	}

	type_object = PyObject_Type( object );

	if ( type_object == (PyObject *) NULL ) {
		mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"The type object for object %p is NULL.", object );

		return;
	}

	type_string = PyString_AsString( type_object );

	repr_object = PyObject_Repr( object );

	if ( repr_object == (PyObject *) NULL ) {
		mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"The repr object for object %p is NULL.", object );

		return;
	}

	repr_string = PyString_AsString( repr_object );

	mx_info( "--> Python object address = %p : type = '%s', repr = '%s'",
			object, type_string, repr_string );

	Py_DECREF( type_object );
	Py_DECREF( repr_object );

	return;
}

/*========*/

MX_EXPORT PyObject *
MpPrivate_create_record_list_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] = "MpPrivate_create_record_list_cmd()";

	PyObject *result;
	MX_RECORD *record_list;
	MX_LIST_HEAD *list_head_struct;
	signed long record_list_handle;
	mx_status_type mx_status;

	/* Create the record list.  The pointer 'record_list' points
	 * to the first record in the list.
	 */

	record_list = mx_initialize_record_list();

	if ( record_list == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, fname,
		"Unable to create a new record list." );

		return MpError_SetFromMxStatus( NULL, mx_status );
	}

	MX_DEBUG( 2,("mx_initialize_record_list() succeeded."));

	/* If the list head structure is not set up correctly, we cannot
	 * continue.
	 */

	list_head_struct = mx_get_record_list_head_struct( record_list );

	if ( list_head_struct == (MX_LIST_HEAD *) NULL ) {
		(void) mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"List head structure for record list %p is NULL.",
			record_list );
	}

	/* Initialize the program name to '(mp)' to indicate that this
	 * is an Mp script.  Individual programs can override this
	 * at run time.
	 */

	strlcpy( list_head_struct->program_name, "(mp)",
				MXU_PROGRAM_NAME_LENGTH );

	/* We can't pass a C pointer back to Python, so we must convert
	 * the C pointer 'record_list' into an MX record handle which
	 * is an integer number.
	 */

	/* Create a handle for the record list as a whole. */

	mx_status = mx_create_handle( &record_list_handle,
					MpPrivate_global_record_handle_table,
					record_list );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	record_list->handle = record_list_handle;

	list_head_struct->handle_table = MpPrivate_global_record_handle_table;

	MX_DEBUG( 2,("%s: record_list = %p, record_list_handle = %ld",
			fname, record_list, record_list_handle));

	/* Now pass the record list handle back to Python. */

	result = Py_BuildValue( "l", (long) record_list_handle );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_read_database_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record, *record_list;
	char *database_name;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	/* Get the record handle and the name of the MX database to be read. */

	parse_status = PyArg_ParseTuple( args, "ls", &handle, &database_name );

	if ( parse_status == 0 )
		return NULL;

	/* Get the record pointer for the record handle we were passed. */

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Make sure we are using the list head record. */

	record_list = record->list_head;

	MX_DEBUG( 2,("Reading database '%s'", database_name));

	mx_status = mx_read_database_file( record_list, database_name, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	MX_DEBUG( 2,("Database successfully read."));

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_finish_database_initialization_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record, *record_list;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	/* Get the record pointer for the record handle we were passed. */

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Make sure we are using the list head record. */

	record_list = record->list_head;

	mx_status = mx_finish_database_initialization( record_list );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_initialize_hardware_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record, *record_list;
	long handle;
	int inithw_flags, parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "li", &handle, &inithw_flags );

	if ( parse_status == 0 )
		return NULL;

	/* Get the record pointer for the record handle we were passed. */

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Make sure we are using the list head record. */

	record_list = record->list_head;

	mx_status = mx_initialize_hardware( record_list, inithw_flags );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_shutdown_hardware_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record, *record_list;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	/* Get the record pointer for the record handle we were passed. */

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Make sure we are using the list head record. */

	record_list = record->list_head;

	mx_status = mx_shutdown_hardware( record_list );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_resynchronize_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record, *record_list;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	/* Get the record pointer for the record handle we were passed. */

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Make sure we are using the list head record. */

	record_list = record->list_head;

	mx_status = mx_resynchronize_record( record_list );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_create_record_from_description_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *old_record, *new_record, *record_list;
	char *description;
	long old_record_handle, new_record_handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ls",
					&old_record_handle, &description );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle(
					old_record_handle, &old_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Make sure we are using the list head record. */

	record_list = old_record->list_head;

	/* Create the record. */

	mx_status = mx_create_record_from_description( record_list,
						description, &new_record, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Create a handle for the new record. */

	mx_status = mx_create_handle( &new_record_handle,
					MpPrivate_global_record_handle_table,
					new_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Send the handle back to Python. */

	result = Py_BuildValue( "l", new_record_handle );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_finish_record_initialization_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_finish_record_initialization( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_open_hardware_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_open_hardware( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_close_hardware_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_close_hardware( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_delete_record_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_delete_record( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_get_record_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] = "MpPrivate_get_record_cmd()";

	PyObject *result;
	MX_RECORD *record, *record_list;
	char *record_name;
	long record_list_handle, record_handle, mx_error_code;
	int parse_status, quiet_flag;
	mx_status_type mx_status;

	MX_DEBUG( 2,("%s invoked.", fname));

	parse_status = PyArg_ParseTuple( args, "lsi",
			&record_list_handle, &record_name, &quiet_flag );

	if ( parse_status == 0 )
		return NULL;

	/* Get the MX_RECORD pointer for the record list. */

	mx_status = MpPrivate_get_record_from_handle( record_list_handle,
							&record_list );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* See if we can find the named record in the specified record list. */

	record = mx_get_record( record_list, record_name );

	if ( record == NULL ) {
		if ( quiet_flag ) {
			mx_error_code = (MXE_NOT_FOUND | MXE_QUIET);
		} else {
			mx_error_code = MXE_NOT_FOUND;
		}

		mx_status = mx_error( mx_error_code, fname,
		"The record '%s' was not found in the database.",
			record_name );

		return MpError_SetFromMxStatus( NULL, mx_status );
	}

	/* Convert the record pointer to a handle. */

	mx_status = mx_get_handle_from_pointer( &record_handle,
				MpPrivate_global_record_handle_table, record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Pass the record handle back to Python. */

	result = Py_BuildValue( "l", record_list_handle );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_get_next_record_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] = "MpPrivate_get_next_record_cmd()";

	PyObject *result;
	MX_RECORD *record, *next_record;
	long record_handle, next_handle;
	int parse_status;
	mx_status_type mx_status;

	MX_DEBUG( 2,("%s invoked.", fname));

	parse_status = PyArg_ParseTuple( args, "l", &record_handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( record_handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Get the next record in the record list. */

	MX_DEBUG( 2,("%s: record = '%s'", fname, record->name));

	next_record = record->next_record;

	MX_DEBUG( 2,("%s: next_record = '%s'", fname, next_record->name));

	/* Convert the record pointer to a handle. */

	mx_status = mx_get_handle_from_pointer( &next_handle,
			MpPrivate_global_record_handle_table, next_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	MX_DEBUG( 2,("%s: record_handle = %ld, next_handle = %ld",
		fname, record_handle, next_handle));

	/* Pass the record handle back to Tcl. */

	result = Py_BuildValue( "l", next_handle );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_get_record_name_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] = "MpPrivate_get_record_name_cmd()";

	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	MX_DEBUG( 2,("%s invoked.", fname));

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "s", record->name );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_create_description_from_record_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] =
			"MpPrivate_create_description_from_record_cmd()";

	PyObject *result;
	MX_RECORD *record;
	char buffer[1024];
	long handle;
	int parse_status;
	mx_status_type mx_status;

	MX_DEBUG( 2,("%s invoked.", fname));

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_create_description_from_record(
				record, buffer, sizeof(buffer) );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Send the string back to Python. */

	result = Py_BuildValue( "s", buffer );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_get_field_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] = "MpPrivate_get_field_cmd()";

	PyObject *result;
	MX_RECORD *record;
	MX_RECORD_FIELD *record_field;
	char token_buffer[1024];
	long handle;
	int parse_status;
	char *field_name, *ptr;
	mx_status_type mx_status;

	MX_DEBUG( 2,("%s invoked.", fname));

	parse_status = PyArg_ParseTuple( args, "ls", &handle, &field_name );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	if ( record->num_record_fields <= 0 ) {
		result = Py_BuildValue( "s", "Unsupported" );

		return result;
	}

	mx_status = mx_find_record_field( record, field_name, &record_field );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_create_description_from_field( record, record_field,
			token_buffer, sizeof( token_buffer ) - 1 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* There may be an extra space in front if we invoked
	 * mx_create_array_description().
	 */

	ptr = &token_buffer[0];

	if ( *ptr == ' ' ) {
		ptr++;
	}

	/* Send the string back to Python. */

	result = Py_BuildValue( "s", ptr );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_set_field_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] = "MpPrivate_set_field_cmd()";

	PyObject *result;
	mx_status_type ( *token_parser ) ( void *, char *,
				MX_RECORD *, MX_RECORD_FIELD *,
				MX_RECORD_FIELD_PARSE_STATUS * );
	MX_RECORD *record;
	MX_RECORD_FIELD *record_field;
	MX_RECORD_FIELD_PARSE_STATUS temp_parse_status;
	char *token_buffer, *field_name;
	void *value_ptr;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	static char separators[] = MX_RECORD_FIELD_SEPARATORS;

	parse_status = PyArg_ParseTuple( args, "lss",
					&handle, &field_name, &token_buffer );

	if ( parse_status == 0 )
		return NULL;

	MX_DEBUG( 2,("%s invoked.", fname));

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	MX_DEBUG( 2,("%s: field_name = '%s'", fname, field_name));

	if ( record->num_record_fields <= 0 ) {
		result = Py_BuildValue( "s", "Unsupported" );

		return result;
	}

	mx_status = mx_find_record_field( record, field_name, &record_field );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	value_ptr = mx_get_field_value_pointer( record_field );

	mx_status = mx_get_token_parser( record_field->datatype,
						&token_parser );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	MX_DEBUG( 2,("%s: token_buffer = '%s'", fname, token_buffer));

	temp_parse_status.description = token_buffer;
	temp_parse_status.separators = separators;
	temp_parse_status.num_separators = strlen(separators);
	temp_parse_status.start_of_trailing_whitespace = NULL;

	if ( record_field->datatype == MXFT_STRING ) {
		temp_parse_status.max_string_token_length =
				mx_get_max_string_token_length( record_field );
	} else {
		temp_parse_status.max_string_token_length = 0L;
	}

	if ( (record_field->num_dimensions == 0)
	  || ((record_field->datatype == MXFT_STRING)
	     && (record_field->num_dimensions == 1)) ) {

		mx_status = mx_get_next_record_token( &temp_parse_status,
				token_buffer, 1 + strlen(token_buffer) );

		MX_DEBUG( 2,("%s: single token = '%s'", fname, token_buffer));

		mx_status = (*token_parser)( value_ptr,
				token_buffer, NULL, record_field,
				&temp_parse_status );
	} else {
		mx_status = mx_parse_array_description( value_ptr,
				record_field->num_dimensions - 1, NULL,
				record_field, &temp_parse_status, token_parser);
	}

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_get_object_class_name_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] = "MpPrivate_get_object_class_name_cmd()";

	PyObject *result;
	MX_RECORD *record;
	char buffer[80];
	long handle;
	int parse_status;
	mx_status_type mx_status;

	MX_DEBUG( 2,("%s invoked.", fname));

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* For an unknown type of record, the object name is Record. */

	strlcpy( buffer, "Record", sizeof(buffer) );

	/* For many types of records, we override this. */

	switch( record->mx_superclass ) {
	case MXR_INTERFACE:
		switch( record->mx_class ) {
		case MXI_CAMAC:
			strlcpy( buffer, "CAMAC", sizeof(buffer) );
			break;
		case MXI_GPIB:
			strlcpy( buffer, "GPIB", sizeof(buffer) );
			break;
		case MXI_MODBUS:
			strlcpy( buffer, "MODBUS", sizeof(buffer) );
			break;
		case MXI_PORTIO:
			strlcpy( buffer, "Portio", sizeof(buffer) );
			break;
		case MXI_RS232:
			strlcpy( buffer, "RS232", sizeof(buffer) );
			break;
		case MXI_USB:
			strlcpy( buffer, "USB", sizeof(buffer) );
			break;
		case MXI_VME:
			strlcpy( buffer, "VME", sizeof(buffer) );
			break;
		}
		break;
	case MXR_DEVICE:
		switch( record->mx_class ) {
		case MXC_AMPLIFIER:
			strlcpy( buffer, "Amplifier", sizeof(buffer) );
			break;
		case MXC_ANALOG_INPUT:
			strlcpy( buffer, "AnalogInput", sizeof(buffer) );
			break;
		case MXC_ANALOG_OUTPUT:
			strlcpy( buffer, "AnalogOutput", sizeof(buffer) );
			break;
		case MXC_AREA_DETECTOR:
			strlcpy( buffer, "AreaDetector", sizeof(buffer) );
			break;
		case MXC_AUTOSCALE:
			strlcpy( buffer, "Autoscale", sizeof(buffer) );
			break;
		case MXC_DIGITAL_INPUT:
			strlcpy( buffer, "DigitalInput", sizeof(buffer) );
			break;
		case MXC_DIGITAL_OUTPUT:
			strlcpy( buffer, "DigitalOutput", sizeof(buffer) );
			break;
		case MXC_ENCODER:
			strlcpy( buffer, "Encoder", sizeof(buffer) );
			break;
		case MXC_MULTICHANNEL_ANALYZER:
			strlcpy( buffer, "MCA", sizeof(buffer) );
			break;
		case MXC_MULTICHANNEL_ENCODER:
			strlcpy( buffer, "MCE", sizeof(buffer) );
			break;
		case MXC_MULTICHANNEL_SCALER:
			strlcpy( buffer, "MCS", sizeof(buffer) );
			break;
		case MXC_MOTOR:
			strlcpy( buffer, "Motor", sizeof(buffer) );
			break;
		case MXC_PAN_TILT_ZOOM:
			strlcpy( buffer, "PTZ", sizeof(buffer) );
			break;
		case MXC_PULSE_GENERATOR:
			strlcpy( buffer, "PulseGenerator", sizeof(buffer) );
			break;
		case MXC_RELAY:
			strlcpy( buffer, "Relay", sizeof(buffer) );
			break;
		case MXC_SAMPLE_CHANGER:
			strlcpy( buffer, "SampleChanger", sizeof(buffer) );
			break;
		case MXC_SINGLE_CHANNEL_ANALYZER:
			strlcpy( buffer, "SCA", sizeof(buffer) );
			break;
		case MXC_SCALER:
			strlcpy( buffer, "Scaler", sizeof(buffer) );
			break;
		case MXC_TABLE:
			strlcpy( buffer, "Table", sizeof(buffer) );
			break;
		case MXC_TIMER:
			strlcpy( buffer, "Timer", sizeof(buffer) );
			break;
		case MXC_VIDEO_INPUT:
			strlcpy( buffer, "VideoInput", sizeof(buffer) );
			break;
		}
		break;
	case MXR_SCAN:
		strlcpy( buffer, "Scan", sizeof(buffer) );
		break;
	case MXR_VARIABLE:
		strlcpy( buffer, "Variable", sizeof(buffer) );
		break;
	case MXR_SERVER:
		strlcpy( buffer, "Server", sizeof(buffer) );
		break;
	}

	/* Send the string back to Python. */

	result = Py_BuildValue( "s", buffer );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_get_version_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] = "MpPrivate_get_version_cmd()";

	PyObject *result;
	int major, minor, update;
	char *date;
	char buffer[80];

	MX_DEBUG( 2,("%s invoked.", fname));

	major  = mx_get_major_version();
	minor  = mx_get_minor_version();
	update = mx_get_update_version();
	date   = mx_get_version_date();

	snprintf( buffer, sizeof(buffer),
		"%d.%d.%d (%s)", major, minor, update, date );

	/* Send the string back to Python. */

	result = Py_BuildValue( "s", buffer );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_start_c_debugger_cmd( PyObject *self, PyObject *args )
{
	mx_start_debugger(NULL);

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_c_breakpoint_cmd( PyObject *self, PyObject *args )
{
	mx_breakpoint();

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_c_set_debugger_started_flag_cmd( PyObject *self, PyObject *args )
{
	long flag;
	int parse_status;

	parse_status = PyArg_ParseTuple( args, "l", &flag );

	mx_set_debugger_started_flag( flag );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_c_wait_for_debugger_cmd( PyObject *self, PyObject *args )
{
	mx_wait_for_debugger();

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_log_open_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] = "MpPrivate_log_open_cmd()";

	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	MX_DEBUG( 2,("%s invoked.", fname));

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_log_open( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_log_close_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] = "MpPrivate_log_close_cmd()";

	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	MX_DEBUG( 2,("%s invoked.", fname));

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_log_close( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

/*-------------------------------------------------------------------------*/

MX_EXPORT PyObject *
MpPrivate_parse_network_field_id_cmd( PyObject *self, PyObject *args )
{
	char server_name[ MXU_HOSTNAME_LENGTH+1 ];
	char server_arguments[ MXU_SERVER_ARGUMENTS_LENGTH+1 ];
	char record_name[ MXU_RECORD_NAME_LENGTH+1 ];
	char field_name[ MXU_FIELD_NAME_LENGTH+1 ];
	PyObject *result;
	char *network_field_id;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "s", &network_field_id );

	if ( parse_status == 0 )
		return NULL;

	mx_status = mx_parse_network_field_id( network_field_id,
				server_name, sizeof(server_name) - 1,
				server_arguments, sizeof(server_arguments) - 1,
				record_name, sizeof(record_name) - 1,
				field_name, sizeof(field_name) - 1 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "ssss", server_name, server_arguments,
					record_name, field_name );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_connect_to_mx_server_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	long list_head_handle, server_handle;
	char *server_name;
	long server_port, server_flags;
	MX_RECORD *server_record, *list_head_record;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lsll",
		&list_head_handle, &server_name, &server_port, &server_flags );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( list_head_handle,
							&list_head_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	server_record = list_head_record;

	mx_status = mx_connect_to_mx_server( &server_record,
					server_name, server_port, server_flags);

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Create a handle for the new server record. */

	mx_status = mx_create_handle( &server_handle,
					MpPrivate_global_record_handle_table,
					server_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Send the handle back to Python. */

	result = Py_BuildValue( "l", server_handle );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_wait_for_messages_cmd( PyObject *self, PyObject *args )
{
	long list_head_handle;
	MX_RECORD *list_head_record;
	double timeout_in_seconds;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld",
				&list_head_handle, &timeout_in_seconds );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( list_head_handle,
							&list_head_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_network_wait_for_messages( list_head_record, 
						timeout_in_seconds );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_set_network_debug_cmd( PyObject *self, PyObject *args )
{
	static const char cname[] = "set_network_debug";

	long list_head_handle, network_debug;
	MX_RECORD *list_head_record;
	MX_LIST_HEAD *list_head;
	MX_RECORD *current_record;
	MX_NETWORK_SERVER *network_server;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
				&list_head_handle, &network_debug );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( list_head_handle,
							&list_head_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Set the network debug flag in the list header. */

	list_head = list_head_record->record_superclass_struct;

	if ( network_debug ) {
		mx_multi_set_debug_flags(list_head_record, MXF_NETDBG_SUMMARY);
	} else {
		mx_multi_set_debug_flags(list_head_record, 0);
	}

	/* We must also find all of the MX network server records
	 * and set the network debug flags in them.
	 */

	current_record = list_head_record->next_record;

	while ( current_record != list_head_record ) {
		switch( current_record->mx_type ) {
		case MXN_NET_TCPIP:
		case MXN_NET_UNIX:
			network_server = (MX_NETWORK_SERVER *)
					current_record->record_class_struct;

			if ( network_server == (MX_NETWORK_SERVER *) NULL ) {
				MpError_SetString(
					MpPrivateExc_Corrupt_Data_Structure,
					"%s: The MX_NETWORK_SERVER pointer for "
					"record '%s' is NULL.",
						cname, current_record->name );
			}

			if ( network_debug ) {
				network_server->server_flags
					|= MXF_NETWORK_SERVER_DEBUG_SUMMARY;
			} else {
				network_server->server_flags
					&= (~ MXF_NETWORK_SERVER_DEBUG_SUMMARY);
			}
			break;
		default:
			break;
		}

		current_record = current_record->next_record;
	}

	Py_INCREF(Py_None);

	return Py_None;
}

