/*
 * Name:    mp_util.c
 *
 * Purpose: This file contains the code that initializes the underlying
 *          MX library.  It also includes other miscellaneous functions
 *          that do not fit elsewhere.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2002, 2004, 2006, 2008-2009, 2011 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "mp.h"

#include <signal.h>

#include "mx_version.h"
#include "mx_plot.h"

MX_EXPORT PyObject *
MpPrivate_init_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] = "MpPrivate_init_cmd()";

	mx_status_type mx_status;

	/* Initialize the runtime environment for MX. */

	mx_status = mx_initialize_runtime();

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	MX_DEBUG( 2,("%s invoked", fname));

	/* Initialize the drivers for the various MX record types. */

	mx_status = mx_initialize_drivers();

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Create a global handle table for the MX records.  Configure the
	 * table to be incremented in blocks of 100 handles.
	 */

	mx_status = mx_create_handle_table(
					&MpPrivate_global_record_handle_table,
					100, 1 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Create a global handle table for MX_NETWORK_FIELD structures.
	 * Configure the table to be incremented in blocks of 100 handles.
	 */

	mx_status = mx_create_handle_table(
				&MpPrivate_global_network_field_handle_table,
				100, 1 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_get_debug_level_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	int debug_level;

	debug_level = mx_get_debug_level();

	result = Py_BuildValue( "i", debug_level );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_set_debug_level_cmd( PyObject *self, PyObject *args )
{
	int debug_level, parse_status;

	parse_status = PyArg_ParseTuple( args, "i", &debug_level );

	if ( parse_status == 0 )
		return NULL;

	mx_set_debug_level( debug_level );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_plotting_is_enabled_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int plot_enable_value, parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	plot_enable_value = mx_plotting_is_enabled( record );

	result = Py_BuildValue( "i", plot_enable_value );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_set_plot_enable_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int plot_enable_value, parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "li",
					&handle, &plot_enable_value );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_set_plot_enable( record, plot_enable_value );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_set_program_name_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record_list;
	long handle;
	char *program_name;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ls", &handle, &program_name );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record_list );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_set_program_name( record_list, program_name );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_change_filename_prefix_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	char *old_filename, *old_prefix, *new_prefix;
	char new_filename[MXU_FILENAME_LENGTH+1];
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "sss", 
			&old_filename, &old_prefix, &new_prefix );

	if ( parse_status == 0 )
		return NULL;

	mx_status = mx_change_filename_prefix( old_filename,
					old_prefix, new_prefix,
					new_filename, sizeof(new_filename)-1 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "s", new_filename );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_init_exceptions_cmd( PyObject *self, PyObject *args )
{
	PyObject *mp_module_object;
	PyObject *result;
	int parse_status;

	/* Find the Mp module and its dictionary. */

	parse_status = PyArg_ParseTuple( args, "O", &mp_module_object );

	if ( parse_status == 0 )
		return NULL;

	result = MpError_Init( mp_module_object );

	return result;
}

